from pluginero.plugin import Plugin, Plugins, check_plugin, BasePlugin
from pluginero.plugin import ReadyPlugin, PluginGroup
import typing


class Manager:
    def __init__(self, plugins: Plugins):
        self._available_plugins = plugins
        self._groups: typing.Dict[str, PluginGroup] = {}
        for plugin in plugins:
            if not plugin.name:
                raise ValueError("Invalid name for plugin")
            self._groups[plugin.name] = PluginGroup(plugin.name, plugin)

    @property
    def plugins(self) -> Plugins:
        return self._available_plugins

    def __getattr__(self, name: str):
        if name in self.__dict__:
            return self.__dict__[name]
        elif name in self._groups:
            return self._groups[name]
        else:
            raise AttributeError("Unknown attribute")

    def register(self, plugin: Plugin, plugin_name: str):
        if not check_plugin(plugin):
            raise RuntimeError("Bad plugin")
        group_name = plugin.__bases__[0].name
        self._groups[group_name].register(plugin, plugin_name)
