import typing
import enum


class BasePlugin:
    """The most base plugin. All plugins must inherit from it."""

    name = None  # name needed for call this plugin-like plugins.


Plugin = typing.Type[BasePlugin]
Plugins = typing.Tuple[Plugin, ...]
ReadyPlugin = BasePlugin


class PluginGroup:
    def __init__(self, name: str, base: Plugin):
        self._name = name
        self._base = base
        self._plugins: typing.Dict[str, ReadyPlugin] = {}

    def __getattr__(self, name: str) -> typing.Any:
        if name in self.__dict__:
            return self.__dict__[name]

        elif name in self._plugins:
            return self._plugins[name]

        else:
            raise AttributeError("Unknown attribute")

    def register(self, plugin: Plugin, name: str):
        if not issubclass(plugin, self._base):
            raise RuntimeError("This plugin doesn't inherit from base plugin for this group")
        self._plugins[name] = plugin()


def check_plugin(plugin: Plugin) -> bool:
    """Checks plugin that's valid"""
    if issubclass(plugin, BasePlugin):
        return True
    else:
        return False
